unit hiHexaColorPanel;

interface

uses
  Kol,Windows,Share,Debug,Win,Messages;

type
  //PHexagonPoints = ^THexagonPoints;
  THexagonPoints = array [0..5] of TPoint;

  TCellKind = (ckHexagon, ckGrayscale);

  PCell = ^TCell;
  TCell = record
    ck    : TCellKind;
    Color : TColor;
    Name  : String;
    pt    : TPoint;
    Points: THexagonPoints;
  end;

  THIHexaColorPanel = class(THIWin)
  private
    bm: KOL.PBitmap;

    FBlackness: Byte;
    FRadius: Integer;
    FSide  : Integer;
    FMedian: Integer;
    rcHexagon: TRect;

    rcPreview: TRect;
    FColorCells: PList;
    FSelectedCell: PCell;

    FOnSelected: TOnEvent;
    FOnChange  : TOnEvent;
    bTracking  : Boolean;

    procedure SetBlackness(const Value: Byte);
    function GetTrackbarRect: TRect;

    procedure CreateCells;
    procedure HexCellCoords(ptCenter: TPoint; var pts: THexagonPoints);
    function AdjustColor(AColor: TColor): TColor;
    function AdjustCellColor(pc: PCell): TColor;

    function GetColor: TColor;
    procedure SetColor({const} Value: TColor);
    function GetSelected: PCell;
    procedure SetSelected(const Value: PCell);

    procedure SelectCellOf(Kind: TCellKind; First: Boolean);
    procedure Paint(Sender: PControl; DC: HDC);
    property Selected: PCell read GetSelected write SetSelected;
    procedure onKeyDown(Sender: PControl; var Key: Longint; Shift: DWORD);
    procedure onMouseDown(Sender: PControl; var Mouse: TMouseEventData);
    procedure onMouseUp(Sender: PControl; var Mouse: TMouseEventData);
    procedure onMouseMove(Sender: PControl; var Mouse: TMouseEventData);
    procedure onDblClick(Sender: PControl; var Mouse: TMouseEventData);

    function MouseToCell(X, Y: Integer): PCell;
    procedure DoSelected;
    procedure DrawBackground(Sender: PControl; DC: HDC);
  public
    _event_onSelectColor: THI_Event;
    _event_onDblClick: THI_Event;
    _event_onMouseMove: THI_Event;
    _event_onBlackness: THI_Event;

    procedure Init;
    destructor Destroy; override;

    procedure _work_doBlackness(var _Data: TData; Index: word);
    procedure _var_CurrentColor(var _Data: TData; Index: word);
    procedure _var_R(var _Data:TData; Index: word);
    procedure _var_G(var _Data:TData; Index: word);
    procedure _var_B(var _Data:TData; Index: word);
    procedure _var_HexColor(var _Data:TData; Index: word);

    property Blackness : Byte read FBlackness write SetBlackness;
    property Color     : TColor read GetColor write SetColor;
    property OnSelected: TOnEvent read FOnSelected write FOnSelected;
    property OnChange  : TOnEvent read FOnChange write FOnChange;
  end;

function GetHexColor(AColor: TColor): String;


implementation

procedure THIHexaColorPanel._work_doBlackness;
begin
  Blackness := ToInteger(_Data);
  DoSelected;
end;

procedure THIHexaColorPanel._var_CurrentColor;
begin
  dtInteger(_Data,Color);
end;

procedure THIHexaColorPanel._var_R;
begin
  dtInteger(_Data,GetRValue(Color));
end;

procedure THIHexaColorPanel._var_G;
begin
  dtInteger(_Data,GetGValue(Color));
end;

procedure THIHexaColorPanel._var_B;
begin
  dtInteger(_Data,GetBValue(Color));
end;

procedure THIHexaColorPanel._var_HexColor;
begin
  dtString(_Data,GetHexColor(Color));
end;

function GetHexColor(AColor: TColor): String;
begin
  Result := Int2Hex(GetBValue(AColor), 2) + Int2Hex(GetGValue(AColor), 2) + Int2Hex(GetRValue(AColor), 2);
end;

{procedure ColorToCMYK(RGB: TColor; var C, M, Y, K: byte);
begin
  C := 255 - GetRValue(RGB);
  M := 255 - GetGValue(RGB);
  Y := 255 - GetBValue(RGB);
  if C < M then K := C else K := M;
  if Y < K then K := Y;
  if k > 0 then begin
    c := c - k; m := m - k; y := y - k;
  end;
end;}

function ColorToCMYK(RGB: TColor): String;
var
  C, M, Y, K: byte;
begin
  C := 255 - GetRValue(RGB);
  M := 255 - GetGValue(RGB);
  Y := 255 - GetBValue(RGB);
  if C < M then K := C else K := M;
  if Y < K then K := Y;
  if k > 0 then begin
    c := c - k; m := m - k; y := y - k;
  end;
  Result := 'CMYK:('+int2Str(C)+','+
            int2Str(M)+','+
            int2Str(Y)+','+
            int2Str(K)+')';
end;

function ValueToPercent(AMain, APart: Integer): Integer;
begin
  if AMain <> 0 then
    Result := MulDiv(100, APart, AMain)
  else
    Result := 0;
end;

function PercentToValue(AMain, APercent: Integer): Integer;
begin
  Result := MulDiv(AMain, APercent, 100);
end;

function ReduceColor(Color: TColor; Percent: Integer): TColor;
var
  R, G, B: Byte;
  rgb_val: COLORREF;
begin
  rgb_val := Color2RGB(Color);
  R := GetRValue(rgb_val);
  G := GetGValue(rgb_val);
  B := GetBValue(rgb_val);

  dec(R, round((Percent/100) * R) - 1);
  dec(G, round((Percent/100) * G) - 1);
  dec(B, round((Percent/100) * B) - 1);
  Result := RGB(R, G, B);
end;

const
  TrackbarHeight = 9; //!!!!!!!!!!!!

procedure CalcHexCell(ptCenter: TPoint; iRadius: Integer; var pts: THexagonPoints);
var
  iMedian: Integer;
begin
  iMedian := Round(Sqrt(Sqr(iRadius) - Sqr(iRadius div 2))) + 1;

  pts[0].x := ptCenter.x;
  pts[0].y := ptCenter.y + iRadius;

  pts[1].x := ptCenter.x + iMedian;
  pts[1].y := ptCenter.y + iMedian div 2;

  pts[2].x := ptCenter.x + iMedian;
  pts[2].y := ptCenter.y - iMedian div 2;

  pts[3].x := ptCenter.x;
  pts[3].y := ptCenter.y - iRadius;

  pts[4].x := ptCenter.x - iMedian;
  pts[4].y := ptCenter.y - iMedian div 2;

  pts[5].x := ptCenter.x - iMedian;
  pts[5].y := ptCenter.y + iMedian div 2;
end;


{ The CMYK color cube:

     CYAN(0,255,255) ______ Blue(0,0,255)
                    /\  1 /\
                   /  \  /  \
                  / 2  \/  0 \
  Green(0,255,0) <----------- > MAGENTA(255,0,255)
                  \ 3  /\  5 /
                   \  /  \  /
                    \/  4 \/
   YELLOW(255,255,0) ------ Red(255,0,0)
}
type
  TRGB = record
    r, g, b: Byte;
  end;

  TColorEntry = record
    Name: PChar;
    case Boolean of
      True: (R, G, B, reserved: Byte);
      False: (Color: COLORREF);
  end;

{
  CMYCube: array [0..5] of TRGB = (
    (r:   0; g: 255; b: 255), // Cyan
    (r:   0; g:   0; b: 255), // Blue
    (r:   0; g: 255; b:   0), // Green
    (r: 255; g: 255; b: 255), // Magenta
    (r: 255; g: 255; b:   0), // Yellow
    (r: 255; g:   0; b:   0)  // Red
  );}


{ THIHexaColorPanel }

procedure THIHexaColorPanel.HexCellCoords(ptCenter: TPoint; var pts: THexagonPoints);
begin
  pts[0].x := ptCenter.x;
  pts[0].y := ptCenter.y + FRadius;

  pts[1].x := ptCenter.x + FMedian;
  pts[1].y := ptCenter.y + FMedian div 2;

  pts[2].x := ptCenter.x + FMedian;
  pts[2].y := ptCenter.y - FMedian div 2;

  pts[3].x := ptCenter.x;
  pts[3].y := ptCenter.y - FRadius;

  pts[4].x := ptCenter.x - FMedian;
  pts[4].y := ptCenter.y - FMedian div 2;

  pts[5].x := ptCenter.x - FMedian;
  pts[5].y := ptCenter.y + FMedian div 2;
end;


procedure THIHexaColorPanel.Init;
begin
  Control := NewPaintBox(FParent);
  Control.Style := Control.Style or WS_TABSTOP;
  inherited;
  //Control.DoubleBuffered := True;
  FRadius := 6;
  FSide   := 9;
  FMedian := Round(Sqrt(Sqr(FRadius) - Sqr(FRadius div 2))) + 1;
  Control.Width  := 210;
  Control.Height := 195;
  bm := NewBitmap(Control.Width,Control.Height);

  FColorCells := NewList;
  CreateCells;

  Control.onMouseDown := onMouseDown;;
  Control.onMouseUp := onMouseUp;
  Control.OnMouseDblClk := onDblClick;
  Control.onMouseMove := onMouseMove;
  Control.onKeyDown := onKeyDown;
  Control.OnEraseBkgnd := DrawBackground;
  Control.onPaint := Paint;
end;


destructor THIHexaColorPanel.Destroy;
begin
  bm.Free;
  FColorCells.Free;
  inherited;
end;


procedure THIHexaColorPanel.CreateCells;

  function _RGB(r, g, b: Extended): TColor;
  begin
    Result := RGB(Trunc(r), Trunc(g), Trunc(b));
  end;

var
  i: Integer;
  pt, ptCenter: TPoint;
  pts: THexagonPoints;

  Delta: Extended;
  bGrow: Boolean;
  iRow, nRows, iCellsLine: Integer;
  pc: PCell;
  clSave: TColor;
begin
  if FColorCells = nil then
    Exit;

  if Selected = nil then
    clSave := clWhite
  else
    begin
      clSave := Selected^.Color;
      Selected := nil;
    end;
  FColorCells.Clear;

  rcPreview := MakeRect(0, 0, 0, 0);

  // -- Calculate the CMYK Hexagon
  pt := MakePoint((2*FMedian*(FSide)) div 2 + 2, 6);
  rcHexagon.TopLeft := MakePoint(0, 0);

  bGrow := True;
  iCellsLine := FSide;
  nRows := pred(2*FSide);
  Delta := 255 /(FSide-1);
  for iRow := 1 to nRows do
  begin
    for i := 0 to pred(iCellsLine) do
    begin
      ptCenter  := MakePoint(pt.x + 2*FMedian*i, pt.y);
      HexCellCoords(ptCenter, pts);

      New(pc);
      pc^.ck := ckHexagon;
      pc^.pt := ptCenter;
      if iRow <= FSide then
        begin
          if i <= iCellsLine - FSide then
            // Cyan to Greens
            pc^.Color := _RGB(i*delta, 255, 255-(iRow-1)*delta + i*delta)
          else
            // Cyan to Blues
            pc^.Color := _RGB((iRow-1)*delta, 255-(i-iRow+1)*delta, 255)
        end
      else
        begin
          if i <= iCellsLine - FSide then
            // Greens to Yellows
            pc^.Color := _RGB((i+iRow-FSide-1)*delta + delta, 255, i*delta)
          else
            // Yellows to Reds
            pc^.Color := _RGB(255, 255-(i-(nRows-iRow))*delta, (nRows-iRow)*delta)
        end;

      pc^.Points := pts;
      FColorCells.Add(pc);
    end;

    inc(pt.y, FRadius + FMedian div 2 + 1);
    if bGrow then
      begin
        inc(iCellsLine);
        dec(pt.x, FMedian);
      end
    else
      begin
        dec(iCellsLine);
        inc(pt.x, FMedian);
      end;

    if iCellsLine = nRows then
      bGrow := False;
  end;
  rcHexagon.BottomRight := MakePoint((Control.Width + 2*FMedian*(FSide-1)) div 2, pt.y);

  // -- Calculate the Grayscale Gradient bar
  pt := MakePoint(2, pt.y+1);
  i  := Control.Width - 2;
  iCellsLine := 3;
  nRows := (i - pt.x) div iCellsLine;
  Delta := 255/nRows;
  for i := 1 to nRows + 1 do
  begin
    New(pc);
    pc^.ck := ckGrayscale;
    pc^.Color := _RGB(255-(i-1)*Delta, 255-(i-1)*Delta, 255-(i-1)*Delta);
    pts[0] := MakePoint(pt.x + (i-1)*iCellsLine, pt.y);
    pts[1] := MakePoint(pt.x + (i)*iCellsLine, pt.y+15);
    pc^.Points := pts;
    FColorCells.Add(pc);
  end;

  Color := clSave;
end;


procedure THIHexaColorPanel.SetBlackness(const Value: Byte);
begin
  if (FBlackness <> Value) and (Value in [0..100]) then
  begin
    FBlackness := Value;
    Control.Invalidate;
  end;
end;


procedure THIHexaColorPanel.SelectCellOf(Kind: TCellKind; First: Boolean);
var
  i: Integer;
  pc: PCell;
begin
  for i := 0 to pred(FColorCells.Count) do
  begin
    pc := PCell(FColorCells.Items[i]);
    if pc^.ck = Kind then
      if (First and ((i = 0) or ((i > 0) and (PCell(FColorCells.Items[i-1])^.ck <> Kind)))
      or (not First
           and (
             (i = pred(FColorCells.Count))
             or ((i < pred(FColorCells.Count)) and (PCell(FColorCells.Items[i+1])^.ck <> Kind)))
           )
         )
      then
      begin
        Selected := pc;
        Break;
      end;
  end;
end;

procedure THIHexaColorPanel.onKeyDown(Sender: PControl; var Key: Longint; Shift: DWORD);
var
  i: Integer;
begin
  inherited;
  //if not (ssCtrl in Shift) then
  if Shift and MK_CONTROL = 0 then

    if (Selected <> nil) then
    begin
      i := FColorCells.IndexOf(Selected);
      case Key of
        VK_LEFT:
          begin
            if i = 0 then i := pred(FColorCells.Count) else dec(i);
            Selected := PCell(FColorCells.Items[i]);
            DoSelected;
          end;
        VK_RIGHT:
          begin
            if i = pred(FColorCells.Count) then i := 0 else inc(i);
            Selected := PCell(FColorCells.Items[i]);
            DoSelected;
          end;
        VK_UP, VK_DOWN:;

        VK_RETURN:
          if Assigned(Selected) then // Handle Ctrl + Enter
          begin
            _hi_onEvent(_event_onDblClick,Color);
            //DoSelected;
          end;

      end;

      Selected := PCell(FColorCells.Items[i]);

    end else

  else

    case Key of
      VK_RETURN:
        if Assigned(Selected) then // Handle Ctrl + Enter
        begin
          _hi_onEvent(_event_onDblClick,Color);
          //DoSelected;
        end;

      VK_UP:
        if Blackness > 0 then
        begin
          Blackness := Blackness - 1;
          DoSelected;
          _hi_onEvent(_event_onBlackness,Blackness);
        end;

      VK_DOWN:
        if Blackness < 100 then
        begin
          Blackness := Blackness + 1;
          DoSelected;
          _hi_onEvent(_event_onBlackness,Blackness);
        end;

      VK_LEFT, VK_RIGHT:
        if Selected = nil then
          if Key <> VK_LEFT then
            SelectCellOf(ckGrayscale, False)
        else
          begin
            if Selected^.ck = ckGrayscale then
              if (Key <> VK_RIGHT) then
                SelectCellOf(pred(Selected^.ck), False)

            else
              if (Key = VK_LEFT) then
              begin
                SelectCellOf(pred(Selected^.ck), False);
                DoSelected;
              end else begin
                SelectCellOf(succ(Selected^.ck), True);
                DoSelected;
              end;
          end;
    end;
end;

procedure THIHexaColorPanel.onDblClick(Sender: PControl; var Mouse: TMouseEventData);
begin
  _hi_onEvent(_event_onDblClick,Color);
end;

procedure THIHexaColorPanel.onMouseMove(Sender: PControl; var Mouse: TMouseEventData);
var
  rc: TRect;
  c: TColor;
  pc: PCell;
  s: string;
begin
  inherited;
  //   hint
  pc := MouseToCell(Mouse.X, Mouse.Y);
  if pc <> nil then
  begin
    c := AdjustCellColor(pc);
    s := 'RGB:(' + Int2Str(GetRValue(c)) + ',' +
    Int2Str(GetGValue(c)) + ',' +
    Int2Str(GetBValue(c)) + ')' + #13#10 +
    'HEX: ' + GetHexColor(c) + #13#10 +
    ColorToCMYK(c);
    _hi_onEvent(_event_onMouseMove,s);
  end;


  if PtInRect(rcPreview, MakePoint(Mouse.X, Mouse.Y)) then
    //Cursor := crHandPoint
  else
    begin
      //Cursor := crDefault;
      if bTracking then
      begin
        rc := GetTrackbarRect;
        if (rc.Top > Mouse.Y) then Blackness := 0 else
          if (rc.Bottom-TrackbarHeight < Mouse.Y) then Blackness := 100 else
            Blackness := ValueToPercent(rc.Bottom - rc.Top - TrackbarHeight, Mouse.Y - rc.Top)
      end;
    end;
end;

procedure THIHexaColorPanel.onMouseDown(Sender: PControl; var Mouse: TMouseEventData);
var
  pc: PCell;
  rc: TRect;
begin
  inherited;

  if not Control.Focused then
    Control.Focused := True;

  if PtInRect(rcPreview, MakePoint(Mouse.X, Mouse.Y)) then
    DoSelected
  else
    begin
      pc := MouseToCell(Mouse.X, Mouse.Y);
      if pc <> nil then
        begin
          Selected := pc;
          //if ssDouble in Shift then
            DoSelected;
        end
      else
        begin
          rc := GetTrackbarRect;
          if (rc.Top <= Mouse.Y) and (Mouse.Y <= rc.Bottom{-TrackbarHeight})
          and (rc.Left-3 <= Mouse.X) and (Mouse.X <= rc.Right+3) then
          begin
            Blackness := ValueToPercent(
              rc.Bottom - rc.Top, Mouse.Y - rc.Top {- TrackbarHeight});
            bTracking := True;
          end;
        end
    end;
end;

procedure THIHexaColorPanel.onMouseUp(Sender: PControl; var Mouse: TMouseEventData);
begin
  inherited;
  bTracking := False;
end;

function THIHexaColorPanel.MouseToCell(X, Y: Integer): PCell;
var
  i: Integer;
  hr: HRGN;
  bFound: Boolean;
  pc: PCell;
  pts: THexagonPoints;
begin
  Result := nil;
  for i := 0 to pred(FColorCells.Count) do
  begin
    pc := PCell(FColorCells.Items[i]);
    if (pc^.ck = ckHexagon) then
      begin
        CalcHexCell(pc^.pt, FRadius+1, pts);
        hr := CreatePolygonRgn(pts{pc^.Points}, 6, WINDING {?});
        bFound := PtInRegion(hr, X, Y);
        DeleteObject(hr);
      end
    else
      bFound := PtInRect(
        MakeRect(pc^.Points[0].X, pc^.Points[0].Y, pc^.Points[1].X, pc^.Points[1].Y),
        MakePoint(X, Y)
      );

    if bFound then
    begin
      Result := pc;
      Break;
    end;
  end;
end;

procedure THIHexaColorPanel.Paint(Sender: PControl; DC: HDC);
var
  PS: TPaintStruct;
  i: Integer;
  r: TRect;
  pc: PCell;
  ck: TCellKind;
  pts: THexagonPoints;
  SimilarColors: array [TCellKind] of PCell;
begin

  with bm.Canvas{$ifndef F_P}^{$endif}  do
  try
    Pen.PenWidth   := 1;
    Pen.PenMode    := pmCopy;
    Brush.BrushStyle := bsSolid;

    // Paint cells
    for i := 0 to pred(FColorCells.Count) do
    begin
      pc := PCell(FColorCells.Items[i]);
      with pc^ do
        if ck = ckHexagon then
          begin
            Brush.Color := AdjustCellColor(pc);
            Pen.Color   := Brush.Color;
            Polygon(Points);
          end
        else
          begin
            // Grayscale is not affected by Blackness
           Brush.Color := pc^.Color;
           Pen.Color := Brush.Color;
           Rectangle(Points[0].X, Points[0].Y, Points[1].X, Points[1].Y);
          end;
    end;

    // Paint selections if any.
    // 1. Search all cells for similar colors...
       FillChar(SimilarColors, SizeOf(SimilarColors), 0);
    for i := 0 to pred(FColorCells.Count) do
    begin
      pc := PCell(FColorCells.Items[i]);
      if AdjustCellColor(pc) = AdjustCellColor(Selected) then
        SimilarColors[pc^.ck] := pc;
    end;

    // 2. Display results...
    for ck := Low(TCellKind) to High(TCellKind) do
      if SimilarColors[ck] <> nil then
      begin
        Pen.PenWidth   := 1;
        Pen.PenMode    := pmXor; //pmNotCopy;
        Brush.BrushStyle := bsClear;
        Pen.Color := AdjustCellColor(SimilarColors[ck]);
        with SimilarColors[ck]^ do
          if SimilarColors[ck]^.ck = ckHexagon then
            if SimilarColors[ck] <> Selected then
              Polygon(Points)
            else
              begin
                Pen.PenMode  := pmCopy;
                Pen.Color := clBtnShadow;
                pts := Points;
                for i := 0 to 5 do pts[i] := MakePoint(pts[i].x+1, pts[i].y+1);
                Polygon(pts);
                Pen.Color := clBtnHighLight;
                Polygon(Points);
              end
          else
            // Grayscale is not affected by Blackness
            if SimilarColors[ck] <> Selected then
              Rectangle(Points[0].X-1, Points[0].Y-1, Points[1].X+1, Points[1].Y+1)
            else
              begin
                r := MakeRect(Points[0].X-1, Points[0].Y-1, Points[1].X+1, Points[1].Y+1);
                DrawEdge(bm.Canvas.Handle, r, EDGE_BUMP, BF_RECT);
              end
      end;

    // Paint blackness trackbar right to Hexagon
    // WARNING: THIS TRACKBAR IS NOT COMPATIBLE WITH XP THEMES, IT WILL BE WINDOWS
    // 98 STYLE (THAT'S WHY IT'S PAINTER HAS BEEN COMMENTED)!!!
//    r := GetTrackbarRect;
//    Brush.Color := cl3DLight;
//    Brush.Style := bsSolid;
//    FillRect(r);
//    DrawEdge(bm.Canvas.Handle, r, BDR_SUNKENOUTER, BF_RECT);
//    i := PercentToValue(r.Bottom - r.Top - TrackbarHeight, Blackness);
//    r := MakeRect(r.Left-3, r.Top + i, r.Right+3, r.Top +i + TrackbarHeight);
//    DrawEdge(bm.Canvas.Handle, r, EDGE_RAISED, BF_RECT or BF_MIDDLE or BF_SOFT);


    BitBlt(DC, 0, 0, Control.Width, Control.Height, bm.Canvas.Handle, 0, 0, SRCCOPY);
  finally
    //if Message.DC = 0 then EndPaint(DC, PS);
  end;
end;

function THIHexaColorPanel.GetTrackbarRect: TRect;
begin
  Result := MakeRect(Control.ClientWidth-10, rcHexagon.Top, Control.ClientWidth-5, rcHexagon.Bottom-4);
end;


function THIHexaColorPanel.AdjustColor(AColor: TColor): TColor;
begin
  if FBlackness = 0 then
    Result := AColor
  else
    Result := ReduceColor(AColor, FBlackness);
end;

function THIHexaColorPanel.AdjustCellColor(pc: PCell): TColor;
begin
  if pc = nil then
    Result := clWhite
  else
    if pc^.ck <> ckHexagon then
      Result := pc^.Color
    else
      Result := AdjustColor(pc^.Color);
end;


function THIHexaColorPanel.GetColor: TColor;
begin
  Result := AdjustCellColor(Selected);
end;


procedure THIHexaColorPanel.SetColor({const} Value: TColor);
var
  i: Integer;
  pc: PCell;
begin
  Value := Color2RGB(Value);
  if not Assigned(Selected) or (Assigned(Selected) and (Selected^.Color <> Value)) then
  begin
    Selected := nil;
    for i := 0 to pred(FColorCells.Count) do
    begin
      pc := PCell(FColorCells.Items[i]);
      if AdjustCellColor(pc) = Value then
      begin
        Selected := pc;
        Break;
      end;
    end;
  end;
end;


function THIHexaColorPanel.GetSelected: PCell;
begin
  Result := FSelectedCell;
end;

procedure THIHexaColorPanel.SetSelected(const Value: PCell);
var
  i: Integer;
  pc: PCell;
begin
  if Value <> nil then
    begin
      FSelectedCell := Value;
    end
  else
   for i := 0 to pred(FColorCells.Count) do
    begin
      pc := PCell(FColorCells.Items[i]);
      if pc^.Color = clWhite then
      begin
        FSelectedCell := Value;
        Break;
      end;
    end;

  Control.Invalidate;
end;


procedure THIHexaColorPanel.DoSelected;
begin
  _hi_onEvent(_event_onSelectColor,Color);
end;

procedure THIHexaColorPanel.DrawBackground(Sender: PControl; DC: HDC);
var
  org: TPoint;
  hParent: HWND;
  scrDC: HDC;
begin
  try
    hParent := Sender.Parent.GetWindowHandle;
    scrDC := bm.Canvas.Handle;
    OffsetViewportOrgEx(scrDC, -Sender.Left, -Sender.Top, org);
    SendMessage(hParent, WM_ERASEBKGND, scrDC, 0);
    SendMessage(hParent, WM_PRINTCLIENT, scrDC, PRF_CLIENT);
    SetViewportOrgEx(scrDC, org.x, org.y, nil);
  except

  end;
end;

end.
